const classNameToArray = (cls = "") => cls.split(" ").filter(item => !!item.trim())
/** Boolean 判断有没有这个class */
export const hasClass = (el: HTMLElement | null | undefined, cls: string) => {
   if (!el) return false
   if (cls.includes(" ")) throw new Error("className should not contain space.")
   return el.classList.contains(cls)
}
/**
   添加class  cls:"a b c" or "a" or "a b"
*/
export const addClass = (el: HTMLElement | null | undefined, cls: string) => {
   if (!el) return
   el.classList.add(...classNameToArray(cls))
}

/** 删除class cls:"a b c" or "a" or "a b" */
export const removeClass = (el: HTMLElement | null | undefined, cls: string) => {
   if (!el) return
   el.classList.remove(...classNameToArray(cls))
}
/**
 * 获取el的style值
 *
 *  例：getStyle(document.querySelector("body"),"position") relative
 */
export const getStyle = (el: HTMLElement | null | undefined, styleName: keyof CSSStyleDeclaration): string => {
   if (!el) return ""
   if (styleName === "float") {
      styleName = "cssFloat"
   }
   try {
      const style = el.style[styleName]
      if (style) return style as string
      const computed = document.defaultView?.getComputedStyle(el, "")
      return computed ? (computed[styleName] as string) : ""
   } catch {
      return el.style[styleName] as string
   }
}
/**
 * 给el修改 style
 *
      setStyle(document.querySelector("body"),{left:0,position:"relative"})
      setStyle(document.querySelector("body"),"left","0"})
*/
export const setStyle = (el: HTMLElement | null | undefined, styleName: CSSStyleDeclaration | keyof CSSStyleDeclaration, value?: string) => {
   if (!el) return
   if (typeof styleName === "object") {
      let prop: keyof CSSStyleDeclaration
      for (prop in styleName) {
         setStyle(el, prop, styleName[prop])
      }
   } else {
      typeof value === "string" && (el.style[styleName as any] = value)
   }
}

/** 删除style
 *removeStyle(document.querySelector("body"),{left:"",position:""})
 * removeStyle(document.querySelector("body"),"left")
 */
export const removeStyle = (el: HTMLElement | null | undefined, style: CSSStyleDeclaration | keyof CSSStyleDeclaration) => {
   if (!el) return
   if (typeof style === "object") {
      let prop: keyof CSSStyleDeclaration
      for (prop in style) {
         setStyle(el, prop, "")
      }
   } else {
      setStyle(el, style, "")
   }
}
