/**  http 方法封装 */
import type { axiosApiType } from "@/libs/axios"
import LoadingBar from "@/plugins/loadingBar"
import { showToast } from "vant"

/** ajax 请求 */
export const httpAjax = function <T extends axiosApiType, P extends Parameters<T>>(axiosApi: T, ...nargs: P): Promise<apiResDataType> {
   return new Promise(resolve => {
      axiosApi(...nargs)
         .then(res => {
            resolve(res.data)
         })
         .catch(err => {
            resolve(err)
         })
   })
}
/**错误会提示 ajax 请求 */
export const httpAjaxErrMsg = function <T extends axiosApiType, P extends Parameters<T>>(axiosApi: T, ...nargs: P): Promise<apiResDataType> {
   return new Promise(resolve => {
      axiosApi(...nargs)
         .then(res => {
            const data = res.data as apiResDataType
            if (data.code !== 200) {
               showToast(data.message)
            }
            resolve(res.data)
         })
         .catch(err => {
            showToast(err.message)
            resolve(err)
         })
   })
}
/** 带loadingBar ajax 请求 */
export const httpAjaxLoading = function <T extends axiosApiType, P extends Parameters<T>>(axiosApi: T, ...nargs: P): Promise<apiResDataType> {
   return new Promise(resolve => {
      LoadingBar.loading(true)
      axiosApi(...nargs)
         .then(res => {
            LoadingBar.loading(false)
            resolve(res.data)
         })
         .catch(err => {
            LoadingBar.loading(false)
            resolve(err)
         })
   })
}
/**
 * 带loadingBar并且错误会提示 ajax 请求
 */
export const httpAjaxLoadingErrMsg = function <T extends axiosApiType, P extends Parameters<T>>(axiosApi: T, ...nargs: P): Promise<apiResDataType> {
   return new Promise(resolve => {
      LoadingBar.loading(true)
      axiosApi(...nargs)
         .then(res => {
            LoadingBar.loading(false)
            const data = res.data as apiResDataType
            if (data.code !== 200) {
               showToast(data.message)
            }
            resolve(res.data)
         })
         .catch(err => {
            LoadingBar.loading(false)
            showToast(err.message)
            resolve(err)
         })
   })
}
