export function parseMusicXML(xmlStr: string) {
   console.time("解析xml")
   const xmlParse = new DOMParser().parseFromString(xmlStr, "text/xml")
   const parts = xmlParse.getElementsByTagName("part")
   let firstPart
   for (const part of parts) {
      if (part.children[0]?.tagName === "measure") {
         firstPart = part
         break
      }
   }
   console.log(firstPart)
   const measuresData = firstPart ? parsePart(firstPart) : []
   console.timeEnd("解析xml")
   return measuresData
}

export type measuresType = {
   stem: "down" | "up"
   type: 4
   dot: boolean
   rest: boolean
   beams: ("begin" | "continue" | "end" | "backward hook")[]
}[][]

function parsePart(firstPart: Element): measuresType[] {
   const measuresData = []
   for (const measure of firstPart.children) {
      const notesData = []
      const notes = measure.getElementsByTagName("note")
      let cacheNote = []
      for (const note of notes) {
         // 符杆朝向
         const stem = note.querySelector("stem")?.textContent === "down" ? "down" : "up"
         // 音符值
         const type = note.querySelector("type")?.textContent || "quarter"
         const dot = !!note.querySelector("dot") || false
         // 是否是休止符
         const rest = !!note.querySelector("rest") || false
         // 音符连接线
         const beams = Array.from(note.querySelectorAll("beam")).map(item => {
            return item?.textContent
         })
         const noteData = {
            stem,
            type: formatBeatUnit(type),
            dot,
            rest,
            beams
         }
         // 根据beams信息把多个音符组成一个大类
         if (noteData.beams.length) {
            if (noteData.beams[0] === "begin") {
               cacheNote = []
            }
            cacheNote.push(noteData)
            if (noteData.beams[0] !== "end") {
               continue
            }
            notesData.push(cacheNote)
         } else {
            notesData.push([noteData])
         }
      }
      measuresData.push(notesData)
   }
   console.log(measuresData)
   return measuresData as any
}

export function formatBeatUnit(beatUnit: string) {
   let multiple = 4
   switch (beatUnit) {
      case "1024th":
         multiple = 1024
         break
      case "512th":
         multiple = 512
         break
      case "256th":
         multiple = 256
         break
      case "128th":
         multiple = 128
         break
      case "64th":
         multiple = 64
         break
      case "32nd":
         multiple = 32
         break
      case "16th":
         multiple = 16
         break
      case "eighth":
         multiple = 8
         break
      case "quarter":
         multiple = 4
         break
      case "half":
         multiple = 2
         break
      case "whole":
         multiple = 1
         break
      case "breve":
         multiple = 0.5
         break
      case "long":
         multiple = 0.25
         break
      case "maxima":
         multiple = 0.125
         break
   }

   return multiple
}
